@@ -18,6 +18,9 @@ class UserAuthorize {
   var $reload_action;		//!< ڡɤɬפп
   var $login_fail;		//!< ǧڤ˼Ԥ˿
 
+  // 虜虜ѿˤɬפϡϤʤ
+  var $readonly_user;		//!< ѥ桼
+
   /*!
     \brief 󥹥ȥ饯
   */
@@ -25,6 +28,10 @@ class UserAuthorize {
     $this->reload_action = false;
     $this->login_fail = false;
 
+    // 桼꤬ʤСѿ򤢤ʤ()˽
+    require('config.php');
+    $this->readonly_user = isset($readonly_user) ? $readonly_user : array();
+
     // å򳫻Ϥμ
     session_name('LANGUPLOADSID');
     session_start();
@@ -36,26 +43,41 @@ class UserAuthorize {
     }
 
     // 
-    if (isset($_POST['login_submit']) &&
-	isset($_POST['login_name']) && isset($_POST['login_password'])) {
+    $users = $this->getUserNames();
+    if ((isset($_POST['login_submit']) && isset($_POST['login_name'])) &&
+	(isset($_POST['login_password']) ||
+	 $_POST['login_name'] == $this->readonly_user)) {
       $login_name = $_POST['login_name'];
       if (! $this->login($login_name, $_POST['login_password'])) {
 	return;
       }
       // 
-      $_SESSION['brows_name'] = $_SESSION['login_name'];
+      if ($this->isProjectUser()) {
+	$_SESSION['brows_name'] = $_SESSION['login_name'];
+      } else {
+	// 桼ϿκǽΥ桼Ȥ
+	$_SESSION['brows_name'] = (count($users) > 0) ?
+	  array_shift($users) : '';
+      }
     }
 
     // 桼ѹ
     if (isset($_GET['brows_name'])) {
       // ¸ߤ桼ǧ
-      if (in_array($_GET['brows_name'], $this->getUserNames())) {
+      if (in_array($_GET['brows_name'], $users)) {
 	$_SESSION['brows_name'] = $_GET['brows_name'];
       }
     }
   }
 
 
+  // ץȤΥ桼Ƚꤹ
+  function isProjectUser() {
+    return in_array($_SESSION['login_name'], $this->getUserNames()) ?
+      true : false;
+  }
+
+
   /*!
     \brief 
   */
@@ -66,6 +88,13 @@ class UserAuthorize {
       return false;
     }
 
+    if ($name == $this->readonly_user) {
+      $_SESSION['login_name'] = $name;
+      $this->reload_action = true;
+      $this->login_fail = false;
+      return true;
+    }
+
     // ե뤫ѥʸ
     $this->login_fail = true;
     $fp = @fopen($_SESSION['users_file'], 'r');
@@ -196,11 +225,6 @@ class LangUpload {
   var $rev;			//!< ӥֹ
   var $key;			//!< ɽؼ
   var $read_mtime;		//!< ɻ
-
-  // !!! ɬפ˱ơѿ
-  // !!! ¿ʬ
-  var $files_mtime;		//!< եι
-
   var $unread_pages;		//!< ̤ɥե
 
 
@@ -214,7 +238,7 @@ class LangUpload {
 
     // ե뤫νͤɤ߽Ф
     if (! isset($_SESSION['admin_mail_address'])) {
-      require_once('config.php');
+      require('config.php');
 
       $_SESSION['admin_mail_address'] =
 	isset($admin_mail_address) ? $admin_mail_address : "no mail address";
@@ -235,7 +259,6 @@ class LangUpload {
     $this->send_comment = false;
     $this->program_updated = false;
     $this->read_mtime = array();
-    $this->files_mtime = array();
     $this->unread_pages = array();
 
     // եι
@@ -402,9 +425,6 @@ class LangUpload {
 
   /*!
     \brief ֹ󥯤κ
-
-    \todo ̤ɥȤΤֹϡ
-    \todo ץब¸ߤСʬ褦ˤ
   */
   function createNoLink($chapter, $no_label = '%d-%d') {
     $out = '';
@@ -417,7 +437,15 @@ class LangUpload {
 
     for ($i = 0; $i < $n; ++$i) {
       $index = $i + 1;
-      $out .= '<td><a href=?no='. $index. '>&nbsp;'. $index. '</a></td>';
+      $key = $chapter. '-'. $index;
+      $back_color = (isset($this->programs_exist[$key]) ||
+		     isset($this->comments_exist[$key])) ? 'white' : '#d0d0d0';
+
+      if ($index == $no) {
+	$out .= '<td bgcolor='. $back_color. '><strong>&nbsp;'. $index. '</strong></a></td>';
+      } else {
+	$out .= '<td bgcolor='. $back_color. '><a href=?no='. $index. '><strong>&nbsp;'. $index. '</strong></a></td>';
+      }
     }
     $out .= '</tr></table>';
     return $out;
@@ -430,7 +458,6 @@ class LangUpload {
     \todo Ȥͽ꤬ʤС
 
     \todo Smarty ΰ̣礤̵...Ĥ
-    \todo ̤ɥ󥯤ξϤʬ褦ɽ롣Ĥơ֥Ǥ狼뤫...
   */
   function createChapterTable($brows_name,
 			      $chapter_view = 'Chapter. %d',
@@ -448,8 +475,8 @@ class LangUpload {
     foreach ($_SESSION['problems_num'] as $chapter_array) {
       // 󥯺
       $ret .= '<tr height=28><td>&nbsp;';
-      $ret .= '<a href=chapterView.php?chapter='. $chapter_no. '>'.
-	sprintf($chapter_view, $chapter_no). '</a>';
+      $ret .= '<a href=chapterView.php?chapter='. $chapter_no. '><strong>'.
+	sprintf($chapter_view, $chapter_no). '</strong></a>';
 
       // Ľκ
       $exists_num = $exists[0][$chapter_no -1];
@@ -507,7 +534,7 @@ class LangUpload {
     for ($i = 0; $i < $_SESSION['problems_num'][$chapter - 1]; ++$i) {
       $no = $i + 1;
       $ref_front = '<a href=programView.php?chapter='. $chapter. '&no='. $no;
-      $out .= '<tr><td align=center>'. $ref_front. '>&nbsp;'. $chapter. '-'. $no. '&nbsp;</a></td>';
+      $out .= '<tr><td align=center>'. $ref_front. '><strong>&nbsp;'. $chapter. '-'. $no. '&nbsp;</strong></a></td>';
       $key = $chapter. '-'. ($i + 1);
 
       // ץ¸ɽ
@@ -515,7 +542,7 @@ class LangUpload {
       if (isset($this->programs_exist[$key])) {
 	$rev = $this->programs_exist[$key];
 	$mtime = $this->programs_mtime[$key. '-'. $rev];
-	$out .= $ref_front. '>&nbsp;'. date('Y-m-d', $mtime). '&nbsp;</a>';
+	$out .= $ref_front. '><strong>&nbsp;'. date('Y-m-d', $mtime). '&nbsp;</strong></a>';
       } else {
 	$out .= '&nbsp;';
       }
@@ -526,7 +553,7 @@ class LangUpload {
       if (isset($this->comments_exist[$key])) {
 	$rev = $this->comments_exist[$key];
 	$mtime = $this->comments_mtime[$key. '-'. $rev];
-	$out .= $ref_front. '#comment>&nbsp;'. date('Y-m-d', $mtime). '&nbsp;</a>';
+	$out .= $ref_front. '#comment><strong>&nbsp;'. date('Y-m-d', $mtime). '&nbsp;</strong></a>';
       } else {
 	$out .= '&nbsp;';
       }
@@ -554,18 +581,29 @@ class LangUpload {
       return $out;
     }
     $rev = $this->rev;
+
+    // !!! path ä̾ѹ롢
     $path = 'data/'. $brows_name. '/p'. $this->key. '-'. $rev. '.txt';
 
     // !!! enscript ʤȤե⡼ɤΤȤб
     $cmd = $_SESSION['enscript_bin'].
       ' --highlight=cpp --color --language html --toc --output - '. $path;
-    $stdout = shell_exec($cmd);
+    $stdout = @shell_exec($cmd);
     $code = mb_detect_encoding($stdout, "auto", true);
     $converted = ($code != false) ?
       mb_convert_encoding($stdout, 'EUC-JP', $code) : $stdout;
-    ereg('(<PRE>.+</PRE>)', $converted, $prog);
 
-    $out .= $prog[1];
+    // !!! preg_match ֤
+    if (ereg('(<PRE>.+</PRE>)', $converted, $prog)) {
+      $out .= $prog[1];
+    } else {
+      $lines = @file($path);
+      if (is_array($lines)) {
+	$out .= '<pre>'. join($lines). '</pre>';
+      } else {
+	$out .= $no_program_message;
+      }
+    }
     return $out;
   }
 
@@ -671,11 +709,11 @@ class LangUpload {
   function createInternalLink($chapter, $no) {
     $out = '<a href=main.php><strong>Main Page</strong></a>';
     if (($chapter > 0) && ($no <= 0)) {
-      $out .= ' &gt;&gt; <strong>Chapter Page</strong>';
+      $out .= ' <font size=-1><strong>&gt;&gt;</strong></font> <strong>Chapter Page</strong>';
     }
     if (($chapter > 0) && ($no > 0)) {
-      $out .= ' &gt;&gt; <a href=chapterView.php?chapter='. $chapter.
-	'><strong>Chapter Page</strong></a> &gt;&gt; <strong>Program Page</strong>';
+      $out .= ' <font size=-1><strong>&gt;&gt;</strong></font> <a href=chapterView.php?chapter='. $chapter.
+	'><strong>Chapter Page</strong></a> <font size=-1><strong>&gt;&gt;</strong></font> <strong>Program Page</strong>';
     }
     return $out;
   }
@@ -694,11 +732,11 @@ class LangUpload {
     $out .= '<table class=users width=100%><tr><td>&nbsp;</td>';
     foreach ($users as $each_name) {
       if ($_SESSION['brows_name'] == $each_name) {
-	$out .= '<td class=users><img src=front_left.png></td><td background=front_middle.png>'. $each_name.
-	  '</td><td class=users><img src=front_right.png></td>';
+	$out .= '<td class=users><img src=front_left.png></td><td background=front_middle.png><strong>'. $each_name.
+	  '</strong></td><td class=users><img src=front_right.png></td>';
       } else {
 	$out .= '<td class=users><img src=back_left.png></td><td background=back_middle.png><a class=users href=?brows_name='.
-	  $each_name. '>'. $each_name. '</a></td><td class=users><img src=back_right.png></td>';
+	  $each_name. '><strong>'. $each_name. '</strong></a></td><td class=users><img src=back_right.png></td>';
       }
     }
     $out .= '<td width=100%></td></tr><tr><td><img src=dot.png width=100% height=2</td>';
@@ -848,7 +886,7 @@ class LangUpload {
 
     $fp = @fopen($file, 'r');
     if ($fp == false) {
-      $out .= '<tr><td>No Comments.</td></tr>';
+      $out .= '<tr><td>&nbsp;No Comments.</td></tr>';
     } else {
       flock($fp, LOCK_SH);
       $header = true;
@@ -878,7 +916,8 @@ class LangUpload {
 	  //$code = mb_detect_encoding($line, "auto", true);
 	  //$converted = mb_convert_encoding($line, 'EUC-JP', $code);
 	  $converted = $line;
-	  $out .= htmlspecialchars($converted). '';
+	  $converted = htmlspecialchars($converted). '<br>';
+	  $out .= preg_replace('/ /', '&nbsp;', $converted);
 	}
       }
       fclose($fp);
@@ -918,12 +957,10 @@ class LangUpload {
 	$this->key. '-'. $this->rev. '.txt';
       $fp = fopen($file, 'a');
       if ($fp != false) {
-	$comment_data = $_SESSION['brows_name']. "\t". time(). "\n";
+	$comment_data = $_SESSION['login_name']. "\t". time(). "\n";
 
-	// \n  \n&nbsp; ִ
-	$out .= preg_replace("/\n/", '<br>', $replaced);
-	$comment_data .=
-	  preg_replace("/\n/", "\n ", $_POST['comment_msg']). "\n\n";
+	// ĤΥȤνüϡԣ
+	$comment_data .= rtrim($_POST['comment_msg']). "\n\n";
 	flock($fp, LOCK_EX);
 	fwrite($fp, $comment_data);
 	fclose($fp);
@@ -995,7 +1032,7 @@ class LangUpload {
     }
     $out .= '<table border=1><tr><th>&nbsp;'. $this->key. '&nbsp;';
 
-    $first_index = (isset($this->comments_exist[$this->key])) ? 0 : 1;
+    $first_index = (isset($this->comments_exist[$this->key. '-0'])) ? 0 : 1;
     if ($first_index != $this->rev_max) {
       $out .= 'Rev';
     }
@@ -1004,7 +1041,11 @@ class LangUpload {
     if ($first_index != $this->rev_max) {
       // rev:0 ΥȤСǥå 0 鳫Ϥ
       for ($i = $first_index; $i <= $this->rev_max; ++$i) {
-	$out .= '<td><a href=?rev='. $i. '>&nbsp;'. $i. '</a></td>';
+	if ($i == $this->rev) {
+	  $out .= '<td><strong>&nbsp;'. $i. '</strong></td>';
+	} else {
+	  $out .= '<td><strong><a href=?rev='. $i. '>&nbsp;'. $i. '</strong></a></td>';
+	}
       }
     }
     $out .= '</tr></table>';
@@ -1048,6 +1089,8 @@ class LangUpload {
     if ($dp == false) {
       return;
     }
+
+    $files_mtime = array();
     clearstatcache();
     while (($file = readdir($dp)) !== false) {
       // [pc]X-X-X.txt Ȥե
@@ -1056,12 +1099,12 @@ class LangUpload {
 	$key = $matches[1];
 	$mtime = filemtime($dir_name. '/'. $file);
 
-	if (! isset($this->files_mtime[$key])) {
-	  $this->files_mtime[$key] = $mtime;
+	if (! isset($files_mtime[$key])) {
+	  $files_mtime[$key] = $mtime;
 	} else {
-	  $time = $this->files_mtime[$key];
+	  $time = $files_mtime[$key];
 	  if ($time < $mtime) {
-	    $this->files_mtime[$key] = $mtime;
+	    $files_mtime[$key] = $mtime;
 	  }
 	}
       }
@@ -1079,7 +1122,7 @@ class LangUpload {
     }
 
     // եι郎ɤ⿷Τʤ
-    foreach ($this->files_mtime as $key => $value) {
+    foreach ($files_mtime as $key => $value) {
       if ($key == $now_key) {
 	continue;
       }
@@ -1114,7 +1157,7 @@ class LangUpload {
       $rev = intval($matches[3]);
       $out .= '<a href=programView.php?chapter='. $chapter. '&no='. $no. '&rev='. $rev .'>'. $message. '</a>';
     }
-    return $out;
+    return $out. '<br>';
   }
 
 
